@@ -18,49 +18,36 @@ from __future__ import annotations
 import contextlib
 import functools
 import os
-import platform
 import shutil
 import sqlite3
 import sys
 
 import nox
 
-ON_WINDOWS_CI = "CI" in os.environ and platform.system() == "Windows"
+ON_WINDOWS_CI = "CI" in os.environ and sys.platform.startswith("win32")
 
-nox.needs_version = ">=2024.3.2"
+nox.needs_version = ">=2024.4.15"
 nox.options.default_venv_backend = "uv|virtualenv"
 
-# Skip 'conda_tests' if user doesn't have conda installed
-nox.options.sessions = ["tests", "cover", "lint", "docs"]
-if shutil.which("conda"):
-    nox.options.sessions.append("conda_tests")
-if shutil.which("mamba"):
-    nox.options.sessions.append("mamba_tests")
-if shutil.which("micromamba"):
-    nox.options.sessions.append("micromamba_tests")
+PYPROJECT = nox.project.load_toml("pyproject.toml")
+
+ALL_PYTHONS = [
+    c.split()[-1]
+    for c in PYPROJECT["project"]["classifiers"]
+    if c.startswith("Programming Language :: Python :: 3.")
+]
 
 
-@nox.session
-@nox.parametrize(
-    "python, tox_version",
-    [
-        (python, tox_version)
-        for python in ("3.8", "3.9", "3.10", "3.11", "3.12", "3.13")
-        for tox_version in ("latest", "<4")
-    ],
-)
-def tests(session: nox.Session, tox_version: str) -> None:
+@nox.session(python=ALL_PYTHONS)
+def tests(session: nox.Session) -> None:
     """Run test suite with pytest."""
 
-    coverage_file = (
-        f".coverage.{sys.platform}.{session.python}.tox{tox_version.lstrip('<')}"
-    )
+    coverage_file = f".coverage.{sys.platform}.{session.python}"
 
     session.create_tmp()  # Fixes permission errors on Windows
-    session.install("-r", "requirements-test.txt")
+    session.install(*PYPROJECT["dependency-groups"]["test"], "uv")
     session.install("-e.[tox_to_nox]")
-    if tox_version != "latest":
-        session.install(f"tox{tox_version}")
+    extra_env = {"PYTHONWARNDEFAULTENCODING": "1"}
     session.run(
         "pytest",
         "--cov",
@@ -70,6 +57,7 @@ def tests(session: nox.Session, tox_version: str) -> None:
         *session.posargs,
         env={
             "COVERAGE_FILE": coverage_file,
+            **extra_env,
         },
     )
 
@@ -79,37 +67,45 @@ def tests(session: nox.Session, tox_version: str) -> None:
             con.execute("DELETE FROM file WHERE SUBSTR(path, 2, 1) == ':'")
 
 
-@nox.session(venv_backend="conda")
+@nox.session(venv_backend="conda", default=shutil.which("conda"))
 def conda_tests(session: nox.Session) -> None:
     """Run test suite set up with conda."""
     session.conda_install(
         "--file", "requirements-conda-test.txt", channel="conda-forge"
     )
-    session.install("-e", ".", "--no-deps")
+    session.install("-e.", "--no-deps")
+    # Currently, this doesn't work on Windows either with or without quoting
+    if not sys.platform.startswith("win32"):
+        session.conda_install("requests<99")
     session.run("pytest", *session.posargs)
 
 
-@nox.session(venv_backend="mamba")
+@nox.session(venv_backend="mamba", default=shutil.which("mamba"))
 def mamba_tests(session: nox.Session) -> None:
     """Run test suite set up with mamba."""
     session.conda_install(
         "--file", "requirements-conda-test.txt", channel="conda-forge"
     )
-    session.install("-e", ".", "--no-deps")
+    session.install("-e.", "--no-deps")
+    if not sys.platform.startswith("win32"):
+        session.conda_install("requests<99")
     session.run("pytest", *session.posargs)
 
 
-@nox.session(venv_backend="micromamba")
+@nox.session(venv_backend="micromamba", default=shutil.which("micromamba"))
 def micromamba_tests(session: nox.Session) -> None:
     """Run test suite set up with micromamba."""
     session.conda_install(
         "--file", "requirements-conda-test.txt", channel="conda-forge"
     )
-    session.install("-e", ".", "--no-deps")
+    # Currently, this doesn't work on Windows either with or without quoting
+    session.install("-e.", "--no-deps")
+    if not sys.platform.startswith("win32"):
+        session.conda_install("requests<99")
     session.run("pytest", *session.posargs)
 
 
-@nox.session
+@nox.session(default=False)
 def cover(session: nox.Session) -> None:
     """Coverage analysis."""
     if ON_WINDOWS_CI:
@@ -135,7 +131,7 @@ def lint(session: nox.Session) -> None:
     )
 
 
-@nox.session
+@nox.session(default=False)
 def docs(session: nox.Session) -> None:
     """Build the documentation."""
     output_dir = os.path.join(session.create_tmp(), "output")
@@ -143,8 +139,8 @@ def docs(session: nox.Session) -> None:
         functools.partial(os.path.join, output_dir), ["doctrees", "html"]
     )
     shutil.rmtree(output_dir, ignore_errors=True)
-    session.install("-r", "requirements-test.txt")
-    session.install(".")
+    session.install(*PYPROJECT["dependency-groups"]["docs"])
+    session.install("-e.")
     session.cd("docs")
     sphinx_args = ["-b", "html", "-W", "-d", doctrees, ".", html]
 
@@ -160,7 +156,8 @@ def docs(session: nox.Session) -> None:
 # The following sessions are only to be run in CI to check the nox GHA action
 def _check_python_version(session: nox.Session) -> None:
     if session.python.startswith("pypy"):
-        python_version = session.python[4:]
+        # Drop starting "pypy" and maybe "-"
+        python_version = session.python.lstrip("py-")
         implementation = "pypy"
     else:
         python_version = session.python
@@ -181,34 +178,25 @@ def _check_python_version(session: nox.Session) -> None:
 
 @nox.session(
     python=[
-        "3.8",
-        "3.9",
-        "3.10",
-        "3.11",
-        "3.12",
-        "3.13",
-        "pypy3.9",
-        "pypy3.10",
-    ]
+        *ALL_PYTHONS,
+        "pypy-3.10",
+    ],
+    default=False,
 )
 def github_actions_default_tests(session: nox.Session) -> None:
     """Check default versions installed by the nox GHA Action"""
-    assert sys.version_info[:2] == (3, 11)
+    assert sys.version_info[:2] == (3, 12)
     _check_python_version(session)
 
 
 @nox.session(
     python=[
-        "3.8",
-        "3.9",
-        "3.10",
-        "3.11",
-        "3.12",
-        "3.13",
+        *ALL_PYTHONS,
         "pypy3.8",
         "pypy3.9",
         "pypy3.10",
-    ]
+    ],
+    default=False,
 )
 def github_actions_all_tests(session: nox.Session) -> None:
     """Check all versions installed by the nox GHA Action"""
