@@ -16,19 +16,24 @@ from __future__ import annotations
 
 import contextlib
 import os
+import shutil
+import subprocess
 import sys
 from importlib import metadata
 from pathlib import Path
+from typing import TYPE_CHECKING, Any, Literal
 from unittest import mock
 
 import pytest
 
 import nox
-import nox.__main__
 import nox._options
 import nox.registry
 import nox.sessions
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
 RESOURCES = os.path.join(os.path.dirname(__file__), "resources")
 VERSION = metadata.version("nox")
 
@@ -39,14 +44,14 @@ VERSION = metadata.version("nox")
 os.environ.pop("NOXSESSION", None)
 
 
-def test_main_no_args(monkeypatch):
+def test_main_no_args(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.setattr(sys, "argv", [sys.executable])
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 0
 
         # Call the function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -61,7 +66,7 @@ def test_main_no_args(monkeypatch):
         assert config.posargs == []
 
 
-def test_main_long_form_args():
+def test_main_long_form_args() -> None:
     sys.argv = [
         sys.executable,
         "--noxfile",
@@ -84,7 +89,7 @@ def test_main_long_form_args():
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -102,7 +107,9 @@ def test_main_long_form_args():
         assert config.posargs == []
 
 
-def test_main_no_venv(monkeypatch, capsys):
+def test_main_no_venv(
+    monkeypatch: pytest.MonkeyPatch, capsys: pytest.CaptureFixture[str]
+) -> None:
     # Check that --no-venv overrides force_venv_backend
     monkeypatch.setattr(
         sys,
@@ -118,7 +125,7 @@ def test_main_no_venv(monkeypatch, capsys):
     )
 
     with mock.patch("sys.exit") as sys_exit:
-        nox.__main__.main()
+        nox.main()
         stdout, stderr = capsys.readouterr()
         assert stdout == "Noms, cheddar so good!\n"
         assert (
@@ -129,7 +136,7 @@ def test_main_no_venv(monkeypatch, capsys):
         sys_exit.assert_called_once_with(0)
 
 
-def test_main_no_venv_error():
+def test_main_no_venv_error() -> None:
     # Check that --no-venv can not be set together with a non-none --force-venv-backend
     sys.argv = [
         sys.executable,
@@ -140,10 +147,35 @@ def test_main_no_venv_error():
         "--no-venv",
     ]
     with pytest.raises(ValueError, match="You can not use"):
-        nox.__main__.main()
+        nox.main()
 
 
-def test_main_short_form_args(monkeypatch):
+def test_main_param_force_python(monkeypatch: pytest.MonkeyPatch) -> None:
+    """
+    Check that Python can be forced if something is parametrized over other things.
+    """
+
+    # Check that --no-venv overrides force_venv_backend
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "nox",
+            "--noxfile",
+            os.path.join(RESOURCES, "noxfile_parametrize.py"),
+            "--sessions",
+            "check_package_files(7.5.0)",
+            "--force-python",
+            ".".join(str(v) for v in sys.version_info[:2]),
+        ],
+    )
+
+    with mock.patch("sys.exit") as sys_exit:
+        nox.main()
+        sys_exit.assert_called_once_with(0)
+
+
+def test_main_short_form_args(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.setattr(
         sys,
         "argv",
@@ -166,7 +198,7 @@ def test_main_short_form_args(monkeypatch):
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -180,14 +212,14 @@ def test_main_short_form_args(monkeypatch):
         assert config.reuse_venv == "yes"
 
 
-def test_main_explicit_sessions(monkeypatch):
+def test_main_explicit_sessions(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.setattr(sys, "argv", [sys.executable, "-e", "1", "2"])
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 0
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -196,7 +228,9 @@ def test_main_explicit_sessions(monkeypatch):
         assert config.sessions == ["1", "2"]
 
 
-def test_main_explicit_sessions_with_spaces_in_names(monkeypatch):
+def test_main_explicit_sessions_with_spaces_in_names(
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     monkeypatch.setattr(
         sys, "argv", [sys.executable, "-e", "unit tests", "the unit tests"]
     )
@@ -205,7 +239,7 @@ def test_main_explicit_sessions_with_spaces_in_names(monkeypatch):
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -215,7 +249,7 @@ def test_main_explicit_sessions_with_spaces_in_names(monkeypatch):
 
 
 @pytest.mark.parametrize(
-    "var,option,env,values",
+    ("var", "option", "env", "values"),
     [
         ("NOXSESSION", "sessions", "foo", ["foo"]),
         ("NOXSESSION", "sessions", "foo,bar", ["foo", "bar"]),
@@ -237,7 +271,13 @@ def test_main_explicit_sessions_with_spaces_in_names(monkeypatch):
         "multiple_force_pythons",
     ],
 )
-def test_main_list_option_from_nox_env_var(monkeypatch, var, option, env, values):
+def test_main_list_option_from_nox_env_var(
+    monkeypatch: pytest.MonkeyPatch,
+    var: str,
+    option: str,
+    env: str,
+    values: list[str],
+) -> None:
     monkeypatch.setenv(var, env)
     monkeypatch.setattr(sys, "argv", [sys.executable])
 
@@ -246,7 +286,7 @@ def test_main_list_option_from_nox_env_var(monkeypatch, var, option, env, values
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -258,7 +298,7 @@ def test_main_list_option_from_nox_env_var(monkeypatch, var, option, env, values
 
 
 @pytest.mark.parametrize(
-    "options,env,expected",
+    ("options", "env", "expected"),
     [
         (["--default-venv-backend", "conda"], "", "conda"),
         ([], "mamba", "mamba"),
@@ -266,7 +306,12 @@ def test_main_list_option_from_nox_env_var(monkeypatch, var, option, env, values
     ],
     ids=["option", "env", "option_over_env"],
 )
-def test_default_venv_backend_option(monkeypatch, options, env, expected):
+def test_default_venv_backend_option(
+    monkeypatch: pytest.MonkeyPatch,
+    options: list[str],
+    env: str,
+    expected: str,
+) -> None:
     monkeypatch.setenv("NOX_DEFAULT_VENV_BACKEND", env)
     monkeypatch.setattr(sys, "argv", [sys.executable, *options])
     with mock.patch("nox.workflow.execute") as execute:
@@ -274,7 +319,7 @@ def test_default_venv_backend_option(monkeypatch, options, env, expected):
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -283,14 +328,16 @@ def test_default_venv_backend_option(monkeypatch, options, env, expected):
         assert config.default_venv_backend == expected
 
 
-def test_main_positional_args(capsys, monkeypatch):
+def test_main_positional_args(
+    capsys: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> None:
     fake_exit = mock.Mock(side_effect=ValueError("asdf!"))
 
     monkeypatch.setattr(sys, "argv", [sys.executable, "1", "2", "3"])
     with mock.patch.object(sys, "exit", fake_exit), pytest.raises(
         ValueError, match="asdf!"
     ):
-        nox.__main__.main()
+        nox.main()
     _, stderr = capsys.readouterr()
     assert "Unknown argument(s) '1 2 3'" in stderr
     fake_exit.assert_called_once_with(2)
@@ -300,20 +347,20 @@ def test_main_positional_args(capsys, monkeypatch):
     with mock.patch.object(sys, "exit", fake_exit), pytest.raises(
         ValueError, match="asdf!"
     ):
-        nox.__main__.main()
+        nox.main()
     _, stderr = capsys.readouterr()
     assert "Unknown argument(s) '1 2 3'" in stderr
     fake_exit.assert_called_once_with(2)
 
 
-def test_main_positional_with_double_hyphen(monkeypatch):
+def test_main_positional_with_double_hyphen(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.setattr(sys, "argv", [sys.executable, "--", "1", "2", "3"])
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 0
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -322,7 +369,9 @@ def test_main_positional_with_double_hyphen(monkeypatch):
         assert config.posargs == ["1", "2", "3"]
 
 
-def test_main_positional_flag_like_with_double_hyphen(monkeypatch):
+def test_main_positional_flag_like_with_double_hyphen(
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     monkeypatch.setattr(
         sys, "argv", [sys.executable, "--", "1", "2", "3", "-f", "--baz"]
     )
@@ -331,7 +380,7 @@ def test_main_positional_flag_like_with_double_hyphen(monkeypatch):
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(0)
         assert execute.called
 
@@ -340,42 +389,48 @@ def test_main_positional_flag_like_with_double_hyphen(monkeypatch):
         assert config.posargs == ["1", "2", "3", "-f", "--baz"]
 
 
-def test_main_version(capsys, monkeypatch):
+def test_main_version(
+    capsys: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.setattr(sys, "argv", [sys.executable, "--version"])
 
     with contextlib.ExitStack() as stack:
         execute = stack.enter_context(mock.patch("nox.workflow.execute"))
         exit_mock = stack.enter_context(mock.patch("sys.exit"))
-        nox.__main__.main()
+        nox.main()
         _, err = capsys.readouterr()
         assert VERSION in err
         exit_mock.assert_not_called()
         execute.assert_not_called()
 
 
-def test_main_help(capsys, monkeypatch):
+def test_main_help(
+    capsys: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.setattr(sys, "argv", [sys.executable, "--help"])
 
     with contextlib.ExitStack() as stack:
         execute = stack.enter_context(mock.patch("nox.workflow.execute"))
         exit_mock = stack.enter_context(mock.patch("sys.exit"))
-        nox.__main__.main()
+        nox.main()
         out, _ = capsys.readouterr()
         assert "help" in out
         exit_mock.assert_not_called()
         execute.assert_not_called()
 
 
-def test_main_failure(monkeypatch):
+def test_main_failure(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.setattr(sys, "argv", [sys.executable])
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 1
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_once_with(1)
 
 
-def test_main_nested_config(capsys, monkeypatch):
+def test_main_nested_config(
+    capsys: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.setattr(
         sys,
         "argv",
@@ -389,14 +444,16 @@ def test_main_nested_config(capsys, monkeypatch):
     )
 
     with mock.patch("sys.exit") as sys_exit:
-        nox.__main__.main()
+        nox.main()
         stdout, stderr = capsys.readouterr()
         assert stdout == "Noms, cheddar so good!\n"
         assert "Session snack(cheese='cheddar') was successful." in stderr
         sys_exit.assert_called_once_with(0)
 
 
-def test_main_session_with_names(capsys, monkeypatch):
+def test_main_session_with_names(
+    capsys: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.setattr(
         sys,
         "argv",
@@ -410,7 +467,7 @@ def test_main_session_with_names(capsys, monkeypatch):
     )
 
     with mock.patch("sys.exit") as sys_exit:
-        nox.__main__.main()
+        nox.main()
         stdout, stderr = capsys.readouterr()
         assert stdout == "Noms, cheddar so good!\n"
         assert "Session cheese list(cheese='cheddar') was successful." in stderr
@@ -418,12 +475,14 @@ def test_main_session_with_names(capsys, monkeypatch):
 
 
 @pytest.fixture
-def run_nox(capsys, monkeypatch):
-    def _run_nox(*args):
+def run_nox(
+    capsys: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> Callable[..., tuple[int, str, str]]:
+    def _run_nox(*args: str) -> tuple[int, str, str]:
         monkeypatch.setattr(sys, "argv", ["nox", *args])
 
         with mock.patch("sys.exit") as sys_exit:
-            nox.__main__.main()
+            nox.main()
             stdout, stderr = capsys.readouterr()
             returncode = sys_exit.call_args[0][0]
 
@@ -461,7 +520,9 @@ def run_nox(capsys, monkeypatch):
         ),
     ],
 )
-def test_main_with_normalized_session_names(run_nox, normalized_name, session):
+def test_main_with_normalized_session_names(
+    run_nox: Callable[..., tuple[int, str, str]], normalized_name: str, session: str
+) -> None:
     noxfile = os.path.join(RESOURCES, "noxfile_normalization.py")
     returncode, _, stderr = run_nox(f"--noxfile={noxfile}", f"--session={session}")
     assert returncode == 0
@@ -482,14 +543,93 @@ def test_main_with_normalized_session_names(run_nox, normalized_name, session):
         "test(arg=datetime.datetime(1980, 1, 1))",
     ],
 )
-def test_main_with_bad_session_names(run_nox, session):
+def test_main_with_bad_session_names(
+    run_nox: Callable[..., tuple[Any, Any, Any]],
+    session: str,
+) -> None:
     noxfile = os.path.join(RESOURCES, "noxfile_normalization.py")
     returncode, _, stderr = run_nox(f"--noxfile={noxfile}", f"--session={session}")
     assert returncode != 0
     assert session in stderr
 
 
-def test_main_noxfile_options(monkeypatch, generate_noxfile_options):
+py39py310 = pytest.mark.skipif(
+    shutil.which("python3.10") is None or shutil.which("python3.9") is None,
+    reason="Python 3.9 and 3.10 required",
+)
+
+
+@pytest.mark.parametrize(
+    ("sessions", "expected_order"),
+    [
+        (("g", "a", "d"), ("b", "c", "h", "g", "a", "e", "d")),
+        pytest.param(("m",), ("k-3.9", "k-3.10", "m"), marks=py39py310),
+        pytest.param(("n",), ("k-3.10", "n"), marks=py39py310),
+        (("v",), ("u(django='1.9')", "u(django='2.0')", "v")),
+        (("w",), ("u(django='1.9')", "u(django='2.0')", "w")),
+    ],
+)
+def test_main_requires(
+    run_nox: Callable[..., tuple[Any, Any, Any]],
+    sessions: tuple[str, ...],
+    expected_order: tuple[str, ...],
+) -> None:
+    noxfile = os.path.join(RESOURCES, "noxfile_requires.py")
+    returncode, stdout, _ = run_nox(f"--noxfile={noxfile}", "--sessions", *sessions)
+    assert returncode == 0
+    assert tuple(stdout.rstrip("\n").split("\n")) == expected_order
+
+
+def test_main_requires_cycle(run_nox: Callable[..., tuple[Any, Any, Any]]) -> None:
+    noxfile = os.path.join(RESOURCES, "noxfile_requires.py")
+    returncode, _, stderr = run_nox(f"--noxfile={noxfile}", "--session=i")
+    assert returncode != 0
+    # While the exact cycle reported is not unique and is an implementation detail, this
+    # still serves as a regression test for unexpected changes in the implementation's
+    # behavior.
+    assert "Sessions are in a dependency cycle: i -> j -> i" in stderr
+
+
+def test_main_requires_missing_session(
+    run_nox: Callable[..., tuple[Any, Any, Any]],
+) -> None:
+    noxfile = os.path.join(RESOURCES, "noxfile_requires.py")
+    returncode, _, stderr = run_nox(f"--noxfile={noxfile}", "--session=o")
+    assert returncode != 0
+    assert "Session not found: does_not_exist" in stderr
+
+
+def test_main_requires_bad_python_parametrization(
+    run_nox: Callable[..., tuple[Any, Any, Any]],
+) -> None:
+    noxfile = os.path.join(RESOURCES, "noxfile_requires.py")
+    with pytest.raises(ValueError, match="Cannot parametrize requires"):
+        run_nox(f"--noxfile={noxfile}", "--session=q")
+
+
+@pytest.mark.parametrize("session", ["s", "t"])
+def test_main_requires_chain_fail(
+    run_nox: Callable[..., tuple[Any, Any, Any]], session: str
+) -> None:
+    noxfile = os.path.join(RESOURCES, "noxfile_requires.py")
+    returncode, _, stderr = run_nox(f"--noxfile={noxfile}", f"--session={session}")
+    assert returncode != 0
+    assert "Prerequisite session r was not successful" in stderr
+
+
+@pytest.mark.parametrize("session", ["w", "u"])
+def test_main_requries_modern_param(
+    run_nox: Callable[..., tuple[Any, Any, Any]],
+    session: str,
+) -> None:
+    noxfile = os.path.join(RESOURCES, "noxfile_requires.py")
+    returncode, _, _stderr = run_nox(f"--noxfile={noxfile}", f"--session={session}")
+    assert returncode == 0
+
+
+def test_main_noxfile_options(
+    monkeypatch: pytest.MonkeyPatch, generate_noxfile_options: Callable[..., str]
+) -> None:
     noxfile_path = generate_noxfile_options(reuse_existing_virtualenvs=True)
     monkeypatch.setattr(
         sys,
@@ -508,7 +648,7 @@ def test_main_noxfile_options(monkeypatch, generate_noxfile_options):
         honor_list_request.return_value = 0
 
         with mock.patch("sys.exit"):
-            nox.__main__.main()
+            nox.main()
 
         assert honor_list_request.called
 
@@ -518,7 +658,9 @@ def test_main_noxfile_options(monkeypatch, generate_noxfile_options):
         assert config.reuse_venv == "yes"
 
 
-def test_main_noxfile_options_disabled_by_flag(monkeypatch, generate_noxfile_options):
+def test_main_noxfile_options_disabled_by_flag(
+    monkeypatch: pytest.MonkeyPatch, generate_noxfile_options: Callable[..., str]
+) -> None:
     noxfile_path = generate_noxfile_options(reuse_existing_virtualenvs=True)
     monkeypatch.setattr(
         sys,
@@ -538,7 +680,7 @@ def test_main_noxfile_options_disabled_by_flag(monkeypatch, generate_noxfile_opt
         honor_list_request.return_value = 0
 
         with mock.patch("sys.exit"):
-            nox.__main__.main()
+            nox.main()
 
         assert honor_list_request.called
 
@@ -548,7 +690,9 @@ def test_main_noxfile_options_disabled_by_flag(monkeypatch, generate_noxfile_opt
         assert config.reuse_venv == "no"
 
 
-def test_main_noxfile_options_sessions(monkeypatch, generate_noxfile_options):
+def test_main_noxfile_options_sessions(
+    monkeypatch: pytest.MonkeyPatch, generate_noxfile_options: Callable[..., str]
+) -> None:
     noxfile_path = generate_noxfile_options(reuse_existing_virtualenvs=True)
     monkeypatch.setattr(
         sys,
@@ -560,7 +704,7 @@ def test_main_noxfile_options_sessions(monkeypatch, generate_noxfile_options):
         honor_list_request.return_value = 0
 
         with mock.patch("sys.exit"):
-            nox.__main__.main()
+            nox.main()
 
         assert honor_list_request.called
 
@@ -570,7 +714,7 @@ def test_main_noxfile_options_sessions(monkeypatch, generate_noxfile_options):
 
 
 @pytest.fixture
-def generate_noxfile_options_pythons(tmp_path):
+def generate_noxfile_options_pythons(tmp_path: Path) -> Callable[[str, str, str], str]:
     """Generate noxfile.py with test and launch_rocket sessions.
 
     The sessions are defined for both the default and alternate Python versions.
@@ -578,16 +722,18 @@ def generate_noxfile_options_pythons(tmp_path):
     goes into ``nox.options.sessions`` and ``nox.options.pythons``, respectively.
     """
 
-    def generate_noxfile(default_session, default_python, alternate_python):
+    def generate_noxfile(
+        default_session: str, default_python: str, alternate_python: str
+    ) -> str:
         path = Path(RESOURCES) / "noxfile_options_pythons.py"
-        text = path.read_text()
+        text = path.read_text(encoding="utf-8")
         text = text.format(
             default_session=default_session,
             default_python=default_python,
             alternate_python=alternate_python,
         )
         path = tmp_path / "noxfile.py"
-        path.write_text(text)
+        path.write_text(text, encoding="utf-8")
         return str(path)
 
     return generate_noxfile
@@ -598,8 +744,10 @@ python_next_version = f"{sys.version_info.major}.{sys.version_info.minor + 1}"
 
 
 def test_main_noxfile_options_with_pythons_override(
-    capsys, monkeypatch, generate_noxfile_options_pythons
-):
+    capsys: pytest.CaptureFixture[str],
+    monkeypatch: pytest.MonkeyPatch,
+    generate_noxfile_options_pythons: Callable[..., str],
+) -> None:
     noxfile = generate_noxfile_options_pythons(
         default_session="test",
         default_python=python_next_version,
@@ -611,7 +759,7 @@ def test_main_noxfile_options_with_pythons_override(
     )
 
     with mock.patch("sys.exit") as sys_exit:
-        nox.__main__.main()
+        nox.main()
         _, stderr = capsys.readouterr()
         sys_exit.assert_called_once_with(0)
 
@@ -625,8 +773,10 @@ def test_main_noxfile_options_with_pythons_override(
 
 
 def test_main_noxfile_options_with_sessions_override(
-    capsys, monkeypatch, generate_noxfile_options_pythons
-):
+    capsys: pytest.CaptureFixture[str],
+    monkeypatch: pytest.MonkeyPatch,
+    generate_noxfile_options_pythons: Callable[..., str],
+) -> None:
     noxfile = generate_noxfile_options_pythons(
         default_session="test",
         default_python=python_current_version,
@@ -638,7 +788,7 @@ def test_main_noxfile_options_with_sessions_override(
     )
 
     with mock.patch("sys.exit") as sys_exit:
-        nox.__main__.main()
+        nox.main()
         _, stderr = capsys.readouterr()
         sys_exit.assert_called_once_with(0)
 
@@ -652,7 +802,9 @@ def test_main_noxfile_options_with_sessions_override(
 
 
 @pytest.mark.parametrize(("isatty_value", "expected"), [(True, True), (False, False)])
-def test_main_color_from_isatty(monkeypatch, isatty_value, expected):
+def test_main_color_from_isatty(
+    monkeypatch: pytest.MonkeyPatch, isatty_value: bool, expected: bool
+) -> None:
     monkeypatch.delenv("FORCE_COLOR", raising=False)
     monkeypatch.setattr(sys, "argv", [sys.executable])
     with mock.patch("nox.workflow.execute") as execute:
@@ -662,7 +814,7 @@ def test_main_color_from_isatty(monkeypatch, isatty_value, expected):
 
             # Call the main function.
             with mock.patch.object(sys, "exit"):
-                nox.__main__.main()
+                nox.main()
 
             config = execute.call_args[1]["global_config"]
             assert config.color == expected
@@ -677,7 +829,9 @@ def test_main_color_from_isatty(monkeypatch, isatty_value, expected):
         ("--no-color", False),
     ],
 )
-def test_main_color_options(monkeypatch, color_opt, expected):
+def test_main_color_options(
+    monkeypatch: pytest.MonkeyPatch, color_opt: str, expected: bool
+) -> None:
     monkeypatch.delenv("FORCE_COLOR", raising=False)
     monkeypatch.setattr(sys, "argv", [sys.executable, color_opt])
     with mock.patch("nox.workflow.execute") as execute:
@@ -685,20 +839,22 @@ def test_main_color_options(monkeypatch, color_opt, expected):
 
         # Call the main function.
         with mock.patch.object(sys, "exit"):
-            nox.__main__.main()
+            nox.main()
 
         config = execute.call_args[1]["global_config"]
         assert config.color == expected
 
 
-def test_main_color_conflict(capsys, monkeypatch):
+def test_main_color_conflict(
+    capsys: pytest.CaptureFixture[str], monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.setattr(sys, "argv", [sys.executable, "--forcecolor", "--nocolor"])
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 1
 
         # Call the main function.
         with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
+            nox.main()
             exit.assert_called_with(1)
 
     _, err = capsys.readouterr()
@@ -706,26 +862,26 @@ def test_main_color_conflict(capsys, monkeypatch):
     assert "color" in err
 
 
-def test_main_force_python(monkeypatch):
+def test_main_force_python(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.setattr(sys, "argv", ["nox", "--force-python=3.11"])
     with mock.patch("nox.workflow.execute", return_value=0) as execute:
         with mock.patch.object(sys, "exit"):
-            nox.__main__.main()
+            nox.main()
         config = execute.call_args[1]["global_config"]
     assert config.pythons == config.extra_pythons == ["3.11"]
 
 
-def test_main_reuse_existing_virtualenvs_no_install(monkeypatch):
+def test_main_reuse_existing_virtualenvs_no_install(
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     monkeypatch.setattr(sys, "argv", ["nox", "-R"])
     with mock.patch("nox.workflow.execute", return_value=0) as execute:
         with mock.patch.object(sys, "exit"):
-            nox.__main__.main()
+            nox.main()
         config = execute.call_args[1]["global_config"]
-    assert (
-        config.reuse_existing_virtualenvs
-        and config.no_install
-        and config.reuse_venv == "yes"
-    )
+    assert config.reuse_existing_virtualenvs
+    assert config.no_install
+    assert config.reuse_venv == "yes"
 
 
 @pytest.mark.parametrize(
@@ -740,12 +896,12 @@ def test_main_reuse_existing_virtualenvs_no_install(monkeypatch):
     ],
 )
 def test_main_noxfile_options_with_ci_override(
-    monkeypatch,
-    generate_noxfile_options,
-    should_set_ci_env_var,
-    noxfile_option_value,
-    expected_final_value,
-):
+    monkeypatch: pytest.MonkeyPatch,
+    generate_noxfile_options: Callable[..., str],
+    should_set_ci_env_var: bool,
+    noxfile_option_value: bool | None,
+    expected_final_value: bool,
+) -> None:
     monkeypatch.delenv("CI", raising=False)  # make sure we have a clean environment
     if should_set_ci_env_var:
         monkeypatch.setenv("CI", "True")
@@ -772,7 +928,7 @@ def test_main_noxfile_options_with_ci_override(
         "nox.tasks.honor_list_request", return_value=0
     ) as honor_list_request:
         with mock.patch("sys.exit"):
-            nox.__main__.main()
+            nox.main()
         config = honor_list_request.call_args[1]["global_config"]
     assert config.error_on_missing_interpreters == expected_final_value
 
@@ -786,11 +942,15 @@ def test_main_noxfile_options_with_ci_override(
         "never",
     ],
 )
-def test_main_reuse_venv_cli_flags(monkeypatch, generate_noxfile_options, reuse_venv):
+@pytest.mark.usefixtures("generate_noxfile_options")
+def test_main_reuse_venv_cli_flags(
+    monkeypatch: pytest.MonkeyPatch,
+    reuse_venv: Literal["yes", "no", "always", "never"],
+) -> None:
     monkeypatch.setattr(sys, "argv", ["nox", "--reuse-venv", reuse_venv])
     with mock.patch("nox.workflow.execute", return_value=0) as execute:
         with mock.patch.object(sys, "exit"):
-            nox.__main__.main()
+            nox.main()
         config = execute.call_args[1]["global_config"]
     assert (
         not config.reuse_existing_virtualenvs
@@ -824,12 +984,12 @@ def test_main_reuse_venv_cli_flags(monkeypatch, generate_noxfile_options, reuse_
     ],
 )
 def test_main_noxfile_options_reuse_venv_compat_check(
-    monkeypatch,
-    generate_noxfile_options,
-    reuse_venv,
-    reuse_existing_virtualenvs,
-    expected,
-):
+    monkeypatch: pytest.MonkeyPatch,
+    generate_noxfile_options: Callable[..., str],
+    reuse_venv: str,
+    reuse_existing_virtualenvs: str | bool | None,
+    expected: str,
+) -> None:
     cmd_args = ["nox", "-l"]
     # CLI Compat Check
     if isinstance(reuse_existing_virtualenvs, str):
@@ -857,6 +1017,104 @@ def test_main_noxfile_options_reuse_venv_compat_check(
         "nox.tasks.honor_list_request", return_value=0
     ) as honor_list_request:
         with mock.patch("sys.exit"):
-            nox.__main__.main()
+            nox.main()
         config = honor_list_request.call_args[1]["global_config"]
     assert config.reuse_venv == expected
+
+
+def test_noxfile_options_cant_be_set() -> None:
+    with pytest.raises(AttributeError, match="reuse_venvs"):
+        nox.options.reuse_venvs = True  # type: ignore[attr-defined]
+
+
+def test_noxfile_options_cant_be_set_long() -> None:
+    with pytest.raises(AttributeError, match="i_am_clearly_not_an_option"):
+        nox.options.i_am_clearly_not_an_option = True  # type: ignore[attr-defined]
+
+
+def test_symlink_orig(monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.chdir(Path(RESOURCES) / "orig_dir")
+    subprocess.run([sys.executable, "-m", "nox", "-s", "orig"], check=True)
+
+
+def test_symlink_orig_not(monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.chdir(Path(RESOURCES) / "orig_dir")
+    res = subprocess.run([sys.executable, "-m", "nox", "-s", "sym"], check=False)
+    assert res.returncode == 1
+
+
+def test_symlink_sym(monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.chdir(Path(RESOURCES) / "sym_dir")
+    subprocess.run([sys.executable, "-m", "nox", "-s", "sym"], check=True)
+
+
+def test_symlink_sym_not(monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.chdir(Path(RESOURCES) / "sym_dir")
+    res = subprocess.run([sys.executable, "-m", "nox", "-s", "orig"], check=False)
+    assert res.returncode == 1
+
+
+def test_noxfile_script_mode(monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.delenv("NOX_SCRIPT_MODE", raising=False)
+    job = subprocess.run(
+        [
+            sys.executable,
+            "-m",
+            "nox",
+            "-f",
+            Path(RESOURCES) / "noxfile_script_mode.py",
+            "-s",
+            "example",
+        ],
+        check=False,
+        capture_output=True,
+        text=True,
+        encoding="utf-8",
+    )
+    print(job.stdout)
+    print(job.stderr)
+    assert job.returncode == 0
+    assert "hello_world" in job.stdout
+
+
+def test_noxfile_no_script_mode(monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.setenv("NOX_SCRIPT_MODE", "none")
+    job = subprocess.run(
+        [
+            sys.executable,
+            "-m",
+            "nox",
+            "-f",
+            Path(RESOURCES) / "noxfile_script_mode.py",
+            "-s",
+            "example",
+        ],
+        check=False,
+        capture_output=True,
+        text=True,
+        encoding="utf-8",
+    )
+    assert job.returncode == 1
+    assert "No module named 'cowsay'" in job.stderr
+
+
+def test_noxfile_script_mode_url_req() -> None:
+    job = subprocess.run(
+        [
+            sys.executable,
+            "-m",
+            "nox",
+            "-f",
+            Path(RESOURCES) / "noxfile_script_mode_url_req.py",
+            "-s",
+            "example",
+        ],
+        check=False,
+        capture_output=True,
+        text=True,
+        encoding="utf-8",
+    )
+    print(job.stdout)
+    print(job.stderr)
+    assert job.returncode == 0
+    assert job.stdout.rstrip() == "2024.10.9"
